"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var utils_1 = require("../../../utils");
var node_vert_glsl_1 = __importDefault(require("../shaders/node.vert.glsl.js"));
var node_frag_glsl_1 = __importDefault(require("../shaders/node.frag.glsl.js"));
var node_1 = require("./common/node");
var POINTS = 3, ATTRIBUTES = 5;
var ANGLE_1 = 0, ANGLE_2 = (2 * Math.PI) / 3, ANGLE_3 = (4 * Math.PI) / 3;
var NodeProgram = /** @class */ (function (_super) {
    __extends(NodeProgram, _super);
    function NodeProgram(gl) {
        var _this = _super.call(this, gl, node_vert_glsl_1.default, node_frag_glsl_1.default, POINTS, ATTRIBUTES) || this;
        // Locations
        _this.angleLocation = gl.getAttribLocation(_this.program, "a_angle");
        // Resolution location
        var resolutionLocation = gl.getUniformLocation(_this.program, "u_resolution");
        if (resolutionLocation === null)
            throw new Error("NodeProgram: error while getting resolutionLocation");
        _this.resolutionLocation = resolutionLocation;
        // Bindings
        gl.enableVertexAttribArray(_this.angleLocation);
        gl.vertexAttribPointer(_this.angleLocation, 1, gl.FLOAT, false, _this.attributes * Float32Array.BYTES_PER_ELEMENT, 16);
        _this.bind();
        return _this;
    }
    NodeProgram.prototype.process = function (data, hidden, offset) {
        var array = this.array;
        var i = offset * POINTS * ATTRIBUTES;
        if (hidden) {
            array[i++] = 0;
            array[i++] = 0;
            array[i++] = 0;
            array[i++] = 0;
            array[i++] = 0;
            return;
        }
        var color = (0, utils_1.floatColor)(data.color);
        array[i++] = data.x;
        array[i++] = data.y;
        array[i++] = data.size;
        array[i++] = color;
        array[i++] = ANGLE_1;
        array[i++] = data.x;
        array[i++] = data.y;
        array[i++] = data.size;
        array[i++] = color;
        array[i++] = ANGLE_2;
        array[i++] = data.x;
        array[i++] = data.y;
        array[i++] = data.size;
        array[i++] = color;
        array[i] = ANGLE_3;
    };
    NodeProgram.prototype.render = function (params) {
        var gl = this.gl;
        var program = this.program;
        gl.useProgram(program);
        gl.uniform2f(this.resolutionLocation, params.width, params.height);
        gl.uniform1f(this.ratioLocation, 1 / Math.pow(params.ratio, params.nodesPowRatio));
        gl.uniform1f(this.scaleLocation, params.scalingRatio);
        gl.uniformMatrix3fv(this.matrixLocation, false, params.matrix);
        gl.drawArrays(gl.TRIANGLES, 0, this.array.length / ATTRIBUTES);
    };
    return NodeProgram;
}(node_1.AbstractNodeProgram));
exports.default = NodeProgram;
